@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 from symfit import variables, Parameter, Fit, D, ODEModel
 import numpy as np
-from symfit.contrib.interactive_guess import InteractiveGuess2D
+from symfit.contrib.interactive_guess import InteractiveGuess
 
 
 # First order reaction kinetics. Data taken from http://chem.libretexts.org/Core/Physical_Chemistry/Kinetics/Rate_Laws/The_Rate_Law
@@ -24,7 +24,7 @@ model_dict = {
 }
 model = ODEModel(model_dict, initial={t: tdata[0], A: concentration_A[0], B: 0})
 
-guess = InteractiveGuess2D(model, A=concentration_A, B=concentration_B, t=tdata, n_points=250)
+guess = InteractiveGuess(model, A=concentration_A, B=concentration_B, t=tdata, n_points=250)
 guess.execute()
 print(guess)
 
